/* 
   $Id: NnmCompare.c,v 1.1.1.1 2005/06/29 22:24:21 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

/*
 *		Compare Feature Vector to dic Vectors
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include "NnmFeature.h"
#include "Nnm.h"
#include "NnmProto.h"

static	int		num_dic;
static 	NnmDicCache	*cache = NULL;
static  int		user_dic = -1;

int	NnmSetDictionaryFile( char *fn, int user)
{
  int i;
  NnmDicCache *cp;

  if (cache == NULL) {
    cache = (NnmDicCache *)malloc(sizeof(NnmDicCache));
    num_dic = 1;
    i = 0;
  } else {
    for (i = 0;i < num_dic;i++) {
	if (cache[i].fn == NULL) break;
    }
    if (i >= num_dic) {
	num_dic++;
	cache = (NnmDicCache *)realloc(cache,sizeof(NnmDicCache)*num_dic);
	i = num_dic-1;
    }
  }
  cp = &cache[i];
  NnmInitDicCache(fn,cp,user);
  if (user) user_dic = i;
  return 0;    
}

int	NnmRemoveDictionaryFile( char *fn )
{
  int i;

  for (i = 0;i < num_dic;i++) {
    if (strcmp(cache[i].fn,fn) == 0) {
      NnmClearDicCache(&cache[i]);
      if (user_dic == i) {
	user_dic = -1;
      }
    }
  }
  return 0;    
}

void NnmRefreshDicCache(void)
{
  int i;

  for (i = 0;i < num_dic;i++) {
      char buf[4096];
      NnmDicCache *cp = &cache[i];

      strcpy(buf,cp->fn);
      NnmClearDicCache(&cache[i]);
      NnmInitDicCache(buf,cp, i == user_dic);
  }
}

int NnmUpdateUserDictionary(long code, NnmFeatureRec *fv)
{
    NnmDicCache *cp;
    int i;
    NnmFeatureDicRec stdfv;

    if (user_dic < 0) return -1;
    cp = &cache[user_dic];
    for (i = 0;i < cp->num;i++) {
	if (cp->body[i].code == code) {
	    break;
	}
    }
    if (i < cp->num) {
	NnmDicCacheRec *rp = &(cp->body[i]);
	NnmFSeek(cp->fd,rp->fptr,SEEK_SET);
	NnmReadStandardFeature(cp->fd, &stdfv);
	NnmUpdateFeatureDic(fv,&stdfv);
	rp->candfv = stdfv.Cand;
	NnmFSeek(cp->fd,rp->fptr,SEEK_SET);
    } else {
	NnmFeatureToFeatureDic(code,fv,&stdfv);
	NnmFSeek(cp->fd,0,SEEK_END);
	NnmAppendDicCache(cp,&stdfv);
    }
    NnmWriteStandardFeature(cp->fd,&stdfv);
    return 0;
}

/*
 *	桼åɵΥ
 *		d = (a-b)^2
 */
static	int	NnmDistanceFV_dir( NnmFeatureRec *fv, NnmFeatureDicRec *dicfv )
{
  int	res, d, x, y;

  res = 0;
  for( d=0; d<8; d++ ){
    for( y=0; y<NnmFV_REGION_Y; y++ ){
      for( x=0; x<NnmFV_REGION_X; x++ ){
	int	v, dr;
#ifdef RDISTANCE
	int vr;
#endif
	
	dr = (d>=4)? d-4: d+4;
	v = (fv->dir_fv[d][y][x] - dicfv->dir_fv[d][y][x]);
#ifdef NNM_WEIGHT
	v = v*v*dicfv->weight[d][y][x];
#else
	v = v*v;
#endif
#ifdef RDISTANCE
	vr = (fv->dir_fv[d][y][x] - dicfv->dir_fv[dr][y][x]);
	vr = vr*vr;

	res += ( v<vr )? v: (v+vr)/2;
#else
	res += v;
#endif

      }
    }
  }
  return res;
}


#define NnmDIR_MAX	8

#define NnmDSCALE	1
#define NnmDIST( a, b )	(((b)>0)? ((a)-(b)):((a)-(b))*NnmDSCALE)
#define NnmABS( a, b )	(((a)>(b))? NnmDIST(a,b):NnmDIST(b,a))

static int sizeDParam = 1210 /*( nearly 10*NNM_WEIGH_STD)*/;
static int strLongParam = 1*NNM_WEIGHT_STD;
static int strShortParam = 1*NNM_WEIGHT_STD;
static int resParam = 1*NNM_WEIGHT_STD;
static int strkDisParam = 7*NNM_WEIGHT_STD;

void NnmSetCandidateParam(int size, int strLong, int strShort,
  int res, int strDis) {
    sizeDParam = size;
    strLongParam = strLong;
    strShortParam = strShort;
    resParam = res;
    strkDisParam = strDis;
}

static	int	NnmCandidateDistance( NnmCandFeatureRec *fv,
				      NnmCandFeatureDicRec *stdfv )
{
  int	res, d, strk_dis, i, j;
  int	XYRatio, absRatio, absRatioX, absRatioY, sizeD;
  int	strklong, strkshort;

  strklong = strkshort = 0;
  strk_dis = NnmABS( fv->num_stroke , stdfv->num_stroke );
  if (strk_dis > NNM_MAX_NSTR_DIS) return -1;
  for( res=d=0; d<NnmDIR_MAX; d++ ){
    int	tmp;

    tmp = NnmABS( fv->dir_stroke[d] , stdfv->dir_stroke[d] );
#ifdef NNM_WEIGHT
    res += tmp*stdfv->dir_stroke_weight[d];
#else
    res += tmp;
#endif
  }
  {
    int	fvX, fvY, stdX, stdY;

    fvX = fv->sizeX; fvY = fv->sizeY;
    stdX= stdfv->sizeX; stdY = stdfv->sizeY;

    XYRatio = MAX( ((fvX*stdY)/(stdX*fvY)), ((stdX*fvY)/(fvX*stdY)) );
    absRatioX = MAX( (fvX/stdX), (stdX/fvX) );
    absRatioY = MAX( (fvY/stdY), (stdY/fvY) );
    absRatio = absRatioX + absRatioY;
  }
  sizeD = XYRatio + absRatio;
  
#ifdef CAND_TEST
  for( d=0; d<NnmDIR_MAX; d++ ){
    for( i=0; i<NnmCAND_SPLIT_NUM; i++ ){
      int	tmp;

      tmp = NnmABS( fv->strkLong[d][i] ,
		    stdfv->strkLong[d][i] );
      strklong += tmp;
#ifdef CAND_TEST2
      for( j=0; j<NnmCAND_SPLIT_NUM; j++ ){
	tmp = NnmABS( fv->strkShort[d][i][j] ,
		      stdfv->strkShort[d][i][j] );
	strkshort += tmp;
      }
#else
      tmp = NnmABS( fv->strkShort[d][i],
		    stdfv->strkShort[d][i] );
      strkshort += tmp;
#endif
    }
  }
  return ( sizeD*sizeDParam + strklong*strLongParam +
    strkshort*strShortParam + res*resParam + strk_dis*strkDisParam );
	// strk_disνŤߤ*20ѹ(01Feb23) 1~4δ֤
#else
  return ( res + strk_dis*3 );	// strk_disνŤߤ*20ѹ(01Feb23) 1~4δ֤
#endif
}

static int NnmGetListOfCandidate( NnmFeatureRec *fv, 
				       NnmCandidateRec cand_set[],
				       int num_cands, FILE *monfp,
				       ulong code )
{
  int			dic,i,ci;
  int			i_cate;
  long			fptr;
  NnmDicCache           *cp;

  ci = 0;
  for (dic = 0;dic < num_dic;dic++) {
    cp = &cache[dic];
    for(i = 0; i < cp->num; i++ ){
      long	dist;
      NnmDicCacheRec *rp;
    
      rp = &(cp->body[i]);
      fptr = rp->fptr;
      if ((dist = NnmCandidateDistance( &(fv->Cand), &(rp->candfv) )) < 0) continue;
#ifdef MONOUT
      if (monfp != NULL) {
	fprintf( monfp,"    code %5x [\033$B%c%c\033(B]: dist=%7d\n",
		 rp->code, (rp->code>>8), (rp->code&0xff), dist );
      }
#endif
      if ( ci<num_cands ){
        int found = 0;

        for( i_cate=0; i_cate<ci; i_cate++ ){
	  if ( cand_set[i_cate].code == rp->code ){
	    found = 1;
	    if (dist < cand_set[i_cate].cand_score) {
		cand_set[i_cate].dicfd = cp->fd;
		cand_set[i_cate].fptr = rp->fptr;
		cand_set[i_cate].cand_score = dist;
	    }
	    break;
	  }
        }
	if (!found) {
	  cand_set[ci].code = rp->code;
	  cand_set[ci].dicfd = cp->fd;
	  cand_set[ci].fptr = rp->fptr;
	  cand_set[ci].cand_score = dist;
	  ci++;
	}
      }else{
        long	maxdist;
        ulong	maxidx;
        int found = 0;
      
        maxidx = -1;
        for( maxdist=i_cate=0; i_cate<num_cands; i_cate++ ){
	  if ( cand_set[i_cate].code == rp->code ){
	    found = 1;
	    if (dist < cand_set[i_cate].cand_score) {
		cand_set[i_cate].dicfd = cp->fd;
		cand_set[i_cate].fptr = rp->fptr;
		cand_set[i_cate].cand_score = dist;
	    }
	    break;
	  }
	  if ( maxdist<cand_set[i_cate].cand_score ){
	    maxdist = cand_set[i_cate].cand_score;
	    maxidx = i_cate;
	  }
        }
	if (!found) {
#ifdef MONOUT
	  if (monfp != NULL) 
	    fprintf( monfp, "MAX=%d \t",maxidx);
#endif
      
	  if ( (maxidx!=-1)&&(maxdist>dist) ){
	    cand_set[maxidx].code = rp->code;
	    cand_set[maxidx].dicfd = cp->fd;
	    cand_set[maxidx].fptr = rp->fptr;
	    cand_set[maxidx].cand_score = dist;
	  }
	}
      }
    }
  }

  return ci > num_cands ? num_cands : ci;
}

static int	SwapCandidate( NnmCandidateRec *cand1,
			       NnmCandidateRec *cand2 )
{
  NnmCandidateRec	tmp;

  memcpy( &tmp,  cand1, sizeof(NnmCandidateRec) );
  memcpy( cand1, cand2, sizeof(NnmCandidateRec) );
  memcpy( cand2, &tmp,  sizeof(NnmCandidateRec) );
  return 0;
}


static int	NnmRecogSorting( NnmCandidateRec *cands, int num )
{
  int	i, flg;

  flg = 1;
  for( ; flg ; ){
    flg = 0;
    for( i=1; i<num; i++ ){
      if ( cands[i-1].recog_score > cands[i].recog_score ){
	SwapCandidate( &(cands[i-1]), &(cands[i]) );
	flg = 1;
      }
    }
  }
  return 0;
}



ulong NnmDetailRecognize( NnmFeatureRec *fv, NnmCandidateRec *cands,
				 int num_cands, FILE *monfp, ulong code )
{
  NnmFeatureDicRec	dicfv;
  int			i;
  
  for( i=0; i<num_cands; i++ ){
    if (cands[i].dicfd < 0) continue;
    NnmFSeek( cands[i].dicfd, cands[i].fptr, SEEK_SET );
    NnmReadStandardFeature( cands[i].dicfd, &dicfv );
    cands[i].recog_score = NnmDistanceFV_dir( fv, &dicfv );
    //    cands[i].recog_score += cands[i].cand_score;  //;褯ʤ(01Mar08)
#ifdef MONOUT
    if ( monfp!=NULL ){
      fprintf( monfp,"  (R)  code %5x [\033$B%c%c\033(B]: dist=%7d\n",
	       dicfv.code, (dicfv.code>>8), (dicfv.code&0xff), 
	       cands[i].recog_score );
    }
#endif
  }
#ifdef MONOUT
  if ( monfp!=NULL ){
    fprintf( monfp, "result code %5x [\033$B%c%c\033(B]: dist=%7d\n", 
	     mincode, (mincode>>8), (mincode&0xff), mindist );
  }
#endif
  NnmRecogSorting( cands, num_cands );
  return cands[0].code;
}

ulong	NnmRecognizeCharacterInfo( NnmRawCharInfoRec *RawCharInfo,
				   NnmCandidateRec *cands, int num_cands,
				   NnmFeatureRec *fv, FILE *monfp, ulong code )
{
  NnmCharInfoRec	charInfo;
  int			i;
  
  for( i=0; i<num_cands; i++ ){
    cands[i].code  = 0;
    cands[i].dicfd = -1;
    cands[i].fptr  = 0;
    cands[i].cand_score = 0x0;
    cands[i].recog_score = 0xffffffff;
  }
  NnmConvertCharInfoToFeature( RawCharInfo, &charInfo, fv );
#ifdef MONOUT
  if ( monfp!=NULL ){
    fprintf( monfp, " Target = %5x [\033$B%c%c\033(B]\n",
	     code, code>>8, code&0xff );
  }
#endif
  num_cands = NnmGetListOfCandidate( fv, cands, num_cands, monfp, code );
  return NnmDetailRecognize( fv, cands, num_cands, monfp, code );
}


ulong	NnmRecognizeCharInfo( NnmRawCharInfoRec *charInfo, 
			      NnmCandidateRec *cands, int num_cands, NnmFeatureRec *fv)
{
  FILE	*fp = NULL;
  int	res;

#ifdef MONOUT
  fp = fopen( "mon.out", "a" );
#endif
  res = NnmRecognizeCharacterInfo( charInfo, cands, num_cands,
				fv, fp, 0x0000 );
#ifdef MONOUT
  fclose( fp );
#endif
  return res;
}
