/* 
   $Id: NnmNormalize.c,v 1.1.1.1 2005/06/29 22:24:21 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include "Nnm.h"
#include "NnmProto.h"

#define RawRANGE(x)	(MAX( MIN((x),NnmCANVAS_DISPLAY_SIZE-1),0 ))
#define RANGE(x)	(MAX( MIN((x),NnmCANVAS_INNER_SIZE-1),0 ))

/*
 * ȥΥץΰ۾ʿ경
 */
void	NnmRawStrokeSmoothing( NnmRawStrokeRec *src,
				       NnmRawStrokeRec *dst )
{
  int	len, idx;
  
  if ( (len=dst->len=src->len)<=0 ) return;
  dst->sample_x[0]     = RawRANGE(src->sample_x[0]);
  dst->sample_y[0]     = RawRANGE(src->sample_y[0]);
  dst->sample_x[len-1] = RawRANGE(src->sample_x[len-1]);
  dst->sample_y[len-1] = RawRANGE(src->sample_y[len-1]);
  for( idx=1; idx<len-1; idx++ ){
    int		x1, x2, y1, y2, x, y;
    int		dx, dy, rx, ry;

    x  = src->sample_x[idx];
    y  = src->sample_y[idx];
    x1 = src->sample_x[idx-1];
    y1 = src->sample_y[idx-1];
    x2 = src->sample_x[idx+1];
    y2 = src->sample_y[idx+1];
    dx = ABS( x-x1 ) + ABS( x-x2 );
    dy = ABS( y-y1 ) + ABS( y-y2 );
    if ( (dx)+(dy) < NnmSMOOTH_THRES ){	/* Smoothing */
      rx = ( 2*x + x1 + x2 )/4;
      ry = ( 2*y + y1 + y2 )/4;
    }else{	/* Noise Remove & Linear Correction */
      rx = ( x1 + x2 )/2;
      ry = ( y1 + y2 )/2;
    }
    dst->sample_x[idx] = RawRANGE(rx);
    dst->sample_y[idx] = RawRANGE(ry);
  }
}

/* ץΤܤƤ */
void	NnmRawStrokeRemoveNearly( NnmRawStrokeRec *src,
					  NnmRawStrokeRec *dst )
{
  int	len, src_idx, dst_idx;

  if ( (len=dst->len=src->len)<=0 ) return;
  dst->sample_x[0]     = src->sample_x[0];
  dst->sample_y[0]     = src->sample_y[0];
  for( src_idx=dst_idx=1; src_idx<len-1; src_idx++ ){
    int		dx, dy;
    
    dx = src->sample_x[src_idx] - dst->sample_x[dst_idx-1];
    dy = src->sample_y[src_idx] - dst->sample_y[dst_idx-1];
    if ( ABS(dx)+ABS(dy) < NnmNOISE_THRES ) continue;
    dst->sample_x[dst_idx] = src->sample_x[src_idx];
    dst->sample_y[dst_idx] = src->sample_y[src_idx];
    dst_idx++;
  }
  dst->sample_x[dst_idx] = src->sample_x[len-1];
  dst->sample_y[dst_idx] = src->sample_y[len-1];
  dst_idx++;
  dst->len = dst_idx;
}

/* ̺︺ܺɸ */
void	NnmRawStrokeResampling( NnmRawStrokeRec *src,
					NnmStrokeRec *dst )
{
  int	len, sumx, sumy, srcidx, dstidx, j;
  int	x, y;
  int	cntup;
  int tmp;

  if ( (len=dst->len=src->len)<=0 ) return;
  
  tmp =  src->sample_x[0]*NnmCANVAS_INNER_SIZE;
  dst->sample_x[0] = tmp/NnmCANVAS_DISPLAY_SIZE;

  tmp =  src->sample_y[0]*NnmCANVAS_INNER_SIZE;
  dst->sample_y[0] = tmp/NnmCANVAS_DISPLAY_SIZE;
  /* ƥץ󥰥졼 */
  cntup = (src->len)/NnmStdSTROKE_LEN + 1;
  
  sumx = sumy = j = 0;
  for( srcidx=dstidx=1; srcidx<len-1; ){
    sumx += src->sample_x[srcidx];
    sumy += src->sample_y[srcidx];
    j++; srcidx++;
    if ( j>=cntup ){	/* ƥץ󥰥졼ʬΥץʿѲ */
      x = sumx*NnmCANVAS_INNER_SIZE;
      x /= (cntup*NnmCANVAS_DISPLAY_SIZE);
      y = sumy*NnmCANVAS_INNER_SIZE;
      y /= (cntup*NnmCANVAS_DISPLAY_SIZE);
      if ( (x==dst->sample_x[dstidx-1])&&
	   (y==dst->sample_y[dstidx-1]) ) continue;
      dst->sample_x[dstidx] = x;
      dst->sample_y[dstidx] = y;
      j = sumx = sumy = 0;
      dstidx++;
    }
  }
  /*  */
  x = src->sample_x[srcidx]*NnmCANVAS_INNER_SIZE;
  x /= NnmCANVAS_DISPLAY_SIZE;;
  y = src->sample_y[srcidx]*NnmCANVAS_INNER_SIZE;
  y /= NnmCANVAS_DISPLAY_SIZE;;
  if ( (x==dst->sample_x[dstidx-1])&&
       (y==dst->sample_y[dstidx-1]) ){
    dst->len = dstidx;
  }else{
    dst->sample_x[dstidx] = x;
    dst->sample_y[dstidx] = y;
    dst->len = dstidx+1;
  }
}



void	NnmNonlinearNormalize( NnmCharInfoRec *charInfo )
{
  int	i, n, prex, prey;
  int	sx, sy, ex, ey, len;
  int	Xhist[NnmCANVAS_INNER_SIZE], sumXhist;
  int	Yhist[NnmCANVAS_INNER_SIZE], sumYhist;
  int	addXhist[NnmCANVAS_INNER_SIZE];
  int	addYhist[NnmCANVAS_INNER_SIZE];
  int	NLtransX[NnmCANVAS_INNER_SIZE];
  int	NLtransY[NnmCANVAS_INNER_SIZE];

  for( i=0; i<NnmCANVAS_INNER_SIZE; i++ ){
    NLtransX[i] = NLtransY[i] = Xhist[i] = Yhist[i] = 0;
  }
  /* ҥȥ */
  for( i=0; i<charInfo->num; i++ ){
    prex = prey = -1;
    len = charInfo->stroke[i].len;
    if ( len==1 ){
      int	x, y;

      x = charInfo->stroke[i].sample_x[0];
      y = charInfo->stroke[i].sample_y[0];
      Xhist[x] += NnmNLFIXCONST;
      Yhist[y] += NnmNLFIXCONST;
    }else{
      for( n=1; n<len ; n++){
	int	x, y, x1, x2, y1, y2;
	int	minx, miny, maxx, maxy;
	
	x1 = charInfo->stroke[i].sample_x[n-1];
	x2 = charInfo->stroke[i].sample_x[n  ];
	y1 = charInfo->stroke[i].sample_y[n-1];
	y2 = charInfo->stroke[i].sample_y[n  ];
	minx = MIN(x1,x2); maxx = MAX(x1,x2);
	miny = MIN(y1,y2); maxy = MAX(y1,y2);
	/*
	 * դʤФʤʤϤ
	 */
	for( x=minx; x<=maxx; x++ ){
	  if ( x!=prex ){
	    Xhist[x]+=NnmNLFIXCONST;
	    prex = x;
	  }
	}
	for( y=miny; y<=maxy; y++ ){
	  if ( y!=prey ){
	    Yhist[y]+=NnmNLFIXCONST;
	    prey = y;
	  }
	}
      }
    }
  }
  /* ܶμ */
  for( sx=0; Xhist[sx]==0 ; sx++ ) ;
  for( ex=NnmCANVAS_INNER_SIZE-1; Xhist[ex]==0 ; ex-- ) ;
  for( sy=0; Yhist[sy]==0 ; sy++ ) ;
  for( ey=NnmCANVAS_INNER_SIZE-1; Yhist[ey]==0 ; ey-- ) ;
  /* ѥҥȥ */
  sumXhist = sumYhist = 0;
  for( i=0; i<NnmCANVAS_INNER_SIZE; i++ ){
    Xhist[i] += ( (sx<=i)&&(i<=ex) )? NnmNL_LINEAR: 0;		/*  */
    Yhist[i] += ( (sy<=i)&&(i<=ey) )? NnmNL_LINEAR: 0;
    sumXhist += Xhist[i];		/* ҥȥ */
    sumYhist += Yhist[i];
    addXhist[i] = (i==0) ? Xhist[i]: Xhist[i]+addXhist[i-1];	/* ѥҥȥ */
    addYhist[i] = (i==0) ? Yhist[i]: Yhist[i]+addYhist[i-1];
  }
  /* ɸѴơ֥ */
  for( i=0; i<NnmCANVAS_INNER_SIZE; i++ ){
    NLtransX[i] =  addXhist[i]*NnmCANVAS_INNER_SIZE;
    NLtransX[i] /= sumXhist;
    NLtransX[i] =  MAX( NLtransX[i], 0 );
    NLtransX[i] =  MIN( NLtransX[i], NnmCANVAS_INNER_SIZE-1 );

    NLtransY[i] =  addYhist[i]*NnmCANVAS_INNER_SIZE;
    NLtransY[i] /= sumYhist;
    NLtransY[i] =  MAX( NLtransY[i], 0 );
    NLtransY[i] =  MIN( NLtransY[i], NnmCANVAS_INNER_SIZE-1 );
  }
  /* ȥѴ */
  for( i=0; i<charInfo->num; i++ ){
    for( n=0; n<charInfo->stroke[i].len ; n++){
      int	x1, y1, x2, y2;
      
      x1 = charInfo->stroke[i].sample_x[n];
      y1 = charInfo->stroke[i].sample_y[n];
      x2 = NLtransX[x1];
      y2 = NLtransY[y1];
      charInfo->stroke[i].sample_x[n] = x2;
      charInfo->stroke[i].sample_y[n] = y2;
    }
  }
  charInfo->width  = ex-sx+1;
  charInfo->height = ey-sy+1;
}


// #define STRK_DIVIDE
#ifdef STRK_DIVIDE
#define STRKDIV		4
static int	NnmStrokeDivide( NnmCharInfoRec *src, 
				 NnmCharInfoRec *dst )
{
  int	src_n, dst_n, num;
  int	src_l, dst_l, len;
  int	dpx, dnx, dpy, dny;
  int	dirp1, dirp2, dirn1, dirn2;
  int	d1, d2;

  num = src->num;
  for( src_n=dst_n=0; src_n<num; src_n++,dst_n++ ){
    NnmStrokeRec	*src_strk, *dst_strk;

    src_strk = &(src->stroke[src_n]);
    dst_strk = &(dst->stroke[dst_n]);
    len = src_strk->len;
    for( src_l=dst_l=0; src_l<len; src_l++ ){
      dst_strk->sample_x[dst_l] = src_strk->sample_x[src_l];
      dst_strk->sample_y[dst_l] = src_strk->sample_y[src_l];

      if ( ( src_l-STRKDIV < 0 )||( src_l+STRKDIV >= len-1) ){
	dst_l++;
	continue;
      }
      dpx = src_strk->sample_x[src_l] - src_strk->sample_x[src_l-STRKDIV];
      dpy = src_strk->sample_y[src_l] - src_strk->sample_y[src_l-STRKDIV];
      dnx = src_strk->sample_x[src_l+STRKDIV] - src_strk->sample_x[src_l];
      dny = src_strk->sample_y[src_l+STRKDIV] - src_strk->sample_y[src_l];
      dirp1 = StrokeDirection1( dpx, dpy );
      dirp2 = StrokeDirection2( dpx, dpy );
      dirn1 = StrokeDirection1( dnx, dny );
      dirn2 = StrokeDirection2( dnx, dny );

      d1 = ( (dirp1!=-1)&&(dirn1!=-1)&&(dirp1!=dirn1) )? 1:0;
      d2 = ( (dirp2!=-1)&&(dirn2!=-1)&&(dirp2!=dirn1) )? 1:0;
      if ( d1||d2 ){
	dst_strk->len = dst_l;
	dst_n++;
	dst_strk = &(dst->stroke[dst_n]);
	dst_l = 0;
	dst_strk->sample_x[dst_l] = src_strk->sample_x[src_l];
	dst_strk->sample_y[dst_l] = src_strk->sample_y[src_l];
      }
      dst_l++;
    }
    dst_strk->len = dst_l;
  }
  dst->num = dst_n;
}

#endif

#ifdef STRK_DIVIDE
int	NnmNormalizeCharInfo( NnmRawCharInfoRec *rawci,
			      NnmCharInfoRec *ci )
{
  int			i, num;
  NnmRawStrokeRec	rstrk1, rstrk2;
  NnmStrokeRec		strk1, strk2;
  NnmCharInfoRec	tmpCharInfo;
  
  if ( (num = ci->num = rawci->num)<=0 ) return -1;
  tmpCharInfo.num = num;
  for( i=0; i<num; i++ ){
    NnmRawStrokeSmoothing( &(rawci->stroke[i]), &rstrk1 );
    NnmRawStrokeRemoveNearly( &rstrk1, &rstrk2 );
    NnmRawStrokeResampling( &rstrk2, &(tmpCharInfo.stroke[i]) );
  }
  NnmStrokeDivide( &tmpCharInfo, ci );
  /* Nonlinear Normalize */
  //  if ( (ci->num == 1)&&(ci->stroke[0].len==1) ) return -1;
  NnmNonlinearNormalize( ci );
  return 0;
}
#else
int	NnmNormalizeCharInfo( NnmRawCharInfoRec *rawci,
			      NnmCharInfoRec *ci )
{
  int			i, num;
  NnmRawStrokeRec	rstrk1, rstrk2;
  
  if ( (num = ci->num = rawci->num)<=0 ) return -1;
  for( i=0; i<num; i++ ){
    NnmRawStrokeSmoothing( &(rawci->stroke[i]), &rstrk1 );
    NnmRawStrokeRemoveNearly( &rstrk1, &rstrk2 );
    NnmRawStrokeResampling( &rstrk2, &(ci->stroke[i]) );
  }
  /* Nonlinear Normalize */
  //  if ( (ci->num == 1)&&(ci->stroke[0].len==1) ) return -1;
  NnmNonlinearNormalize( ci );
  return 0;
}
#endif  
