/* 
   $Id: NnmSample.c,v 1.1.1.1 2005/06/29 22:24:21 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "Nnm.h"
#include "NnmFeature.h"
#include "NnmProto.h"


/* ʸȥ¸ */
int	NnmSampleAppendFile( NnmRawCharInfoRec *rawCharInfo, ulong code, 
			     char *fn )
{
  int		fd;
  uchar		buf[16];
  uchar		num;
  ushort	len;
  int		i, j;

  if ( (fd=NnmFOpen( fn, "a+" ))==-1 ){
    fprintf( stderr, "File Error: Cannot open or create file[%s]\n",fn );
    return -1;
  }
  if ( rawCharInfo->num>NnmMaxSTROKE_NUM ){
    fprintf( stderr, "File Error: Number of Stroke is too much (%d)\n",
	     rawCharInfo->num );
    num = 0;
  }else{
    if ( NnmFWrite( &code, sizeof(code), 1, fd )!=1 ){
      fprintf( stderr,"File Error: Cannot write code in file[%s]\n",fn );
      goto WriteError;
    }
    num = rawCharInfo->num;
    if ( NnmFWrite( &num, sizeof(num), 1, fd )!=1 ){
      fprintf( stderr,"File Error: Cannot write stroke num in file[%s]\n",fn );
      goto WriteError;
    }
  }
  for( i=0; i<num; i++ ){
    len = rawCharInfo->stroke[i].len;
    buf[0] = len & 0xff;
    buf[1] = (len>>8) & 0xff;
    if ( NnmFWrite( buf, 2, 1, fd )!=1 ){
      fprintf( stderr,"File Error: Cannot write stroke length in file[%s]\n",fn );
      goto WriteError;
    }
    for( j=0; j<len; j++ ){
      buf[0] = rawCharInfo->stroke[i].sample_x[j];
      buf[1] = rawCharInfo->stroke[i].sample_y[j];
      if ( NnmFWrite( buf, 2, 1, fd )!=1 ){
	fprintf( stderr,"File Error: Cannot write stroke info in file[%s]\n",fn );
	goto WriteError;
      }
    }
  }
  NnmFClose( fd );
  return 0;
  
 WriteError:
  NnmFClose( fd );
  return -1;
}


/* ʸȥɤ߽Ф */
int	NnmSampleReadFile( NnmRawCharInfoRec *rawCharInfo, ulong *code, ulong infd )
{
  uchar		buf[16];
  uchar		num;
  ushort	len;
  int		i, j, res;

  if ( (res=NnmFRead( code, sizeof(unsigned int), 1, infd ))!=1 ){
#ifdef DEBUG_PRINT
    fprintf(stderr,"File Error: Cannot read sample vector file(code)%d,%d\n",
	    res, code);
#endif
    return -1;
  }
  if ( (res=NnmFRead( &num, 1, 1, infd ))!=1 ){
#ifdef DEBUG_PRINT
    fprintf(stderr,"File Error: Cannot read sample vector file(num)%d,%d\n",
	    res, num);
#endif
    return -1;
  }
  if( num<=0 ) return -1;
  rawCharInfo->num = num;
  for( i=0; i<num; i++ ){
    if ( NnmFRead( buf, sizeof(len), 1, infd )!=1 ){
      fprintf(stderr,"File Error: Cannot read sample vector file(num:%d:len)\n",
	      i);
      return -1;
    }
    len = buf[0] | (buf[1]<<8);
    rawCharInfo->stroke[i].len = len;
    for( j=0; j<len; j++ ){
      if ( NnmFRead( buf, 2, 1, infd )!=1 ){
	fprintf(stderr,"File Error: Cannot read sample vector file(samp)\n");
	return -1;
      }
      rawCharInfo->stroke[i].sample_x[j] = buf[0];
      rawCharInfo->stroke[i].sample_y[j] = buf[1];
    }
  }
  return 0;
}



/* ʸȥcode0֤ */
int	NnmSampleRemoveFile( int infd, long fptr )
{
  int		res;
  long		tmpfptr;
  ulong		code = 0;
  
  tmpfptr = NnmFTell( infd );
  NnmFSeek( infd, fptr, SEEK_SET );
  if ( (res=NnmFWrite( &code, sizeof(ulong), 1, infd ))!=1 ){
#ifdef DEBUG_PRINT
    fprintf(stderr,"File Error: Cannot read sample vector file(code)%d,%d\n",
	    res, code);
#endif
    return -1;
  }
  NnmFSeek( infd, tmpfptr, SEEK_SET );
  
  return 0;
}

