/* 
   $Id: NnmUtil.c,v 1.1.1.1 2005/06/29 22:24:22 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include "Nnm.h"
#include "NnmFeature.h"
#include "NnmProto.h"


int	NnmNumberOfDictionary( char *dicfn )
{
  int size;

  if ((size = NnmFSize(dicfn)) < 0) {
    fprintf(stderr," File Error: Cannot get size of DIC file[%s]\n",
	    dicfn );
    return 0;
  }
  return size/DIC_SIZE;
}

int	NnmInitDicCache( char *dicfn, NnmDicCache *cp, int user)
{
  int			num, i, dicfd;
  NnmDicCacheRec	*ptr;
  NnmFeatureDicRec	stdfv;
  char *mode = "r";

  cp->fn = strdup(dicfn);
  cp->num = num = NnmNumberOfDictionary( dicfn );
  cp->body = ptr = (NnmDicCacheRec *)malloc( num*sizeof(NnmDicCacheRec) );
  if ( ptr==NULL ){
    fprintf( stderr, "Cannot allocate Dic Cache\n");
    exit(1);
  }
  if (user) {
    mode = "r+";
  }
  if ( (dicfd=NnmFOpen(dicfn,mode))==-1 ){
    fprintf(stderr," File Error: Cannot open DIC file[%s]\n",
	    dicfn );
    return -1;
  }
  cp->fd = dicfd;
  
  for( i=0; !NnmFEof(dicfd); i++ ){
    ptr[i].fptr = NnmFTell( dicfd );
    NnmReadStandardFeature( dicfd, &stdfv );
    ptr[i].code = stdfv.code;
    memcpy(&(ptr[i].candfv), &(stdfv.Cand), sizeof(NnmCandFeatureDicRec) );
  }
  return 0;
}

void NnmAppendDicCache(NnmDicCache *cp, NnmFeatureDicRec *stdfv)
{
    NnmDicCacheRec *p;

    if (cp->num == 0) {
	cp->body = (NnmDicCacheRec *)malloc((cp->num+1)*sizeof(NnmDicCacheRec) );
    } else {
	cp->body = (NnmDicCacheRec *)realloc(cp->body,
	  (cp->num+1)*sizeof(NnmDicCacheRec) );
    }
    p = &(cp->body[cp->num]);
    p->fptr = NnmFTell(cp->fd);
    p->code = stdfv->code;
    p->candfv = stdfv->Cand;
    cp->num++;
}

void	NnmClearDicCache(NnmDicCache *cp)
{
    if (cp->fn != NULL) {
	free(cp->fn);
	cp->fn = NULL;
    }
    NnmFClose(cp->fd);
    if (cp->body != NULL) {
	free(cp->body);
	cp->body = NULL;
    }
}
