/* 
   $Id: Nnmfile.c,v 1.1.1.1 2005/06/29 22:24:22 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "Nnm.h"
#include "NnmFeature.h"
#include "NnmProto.h"


int	NnmFOpen( char *fn, char *mode )
{
  int	flag;
    
  if ( strcmp(mode,"r")==0 ){
    flag = O_RDONLY;
  }else if ( strcmp(mode,"r+")==0 ){
    flag = O_RDWR;
  }else if ( strcmp(mode,"w")==0 ){
    flag = O_WRONLY | O_CREAT | O_TRUNC;
  }else if ( strcmp(mode,"w+")==0 ){
    flag = O_RDWR | O_CREAT | O_TRUNC;
  }else if ( strcmp(mode,"a")==0 ){
    flag = O_WRONLY | O_CREAT | O_APPEND;
  }else if ( strcmp(mode,"a+")==0 ){
    flag = O_RDWR | O_CREAT | O_APPEND;
  }else{
    flag = O_RDONLY;
  }
  return open( fn, flag, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP );
}

int	NnmFSize(char *fn)
{
  struct stat st;

  if (stat(fn,&st) < 0) return -1;
  return st.st_size;
}

size_t	NnmFRead( void *ptr, size_t size, size_t nmemb, int fd )
{
  return read( fd, ptr, size*nmemb )/size;
}

  
size_t	NnmFWrite( void *ptr, size_t size, size_t nmemb, int fd )
{
  return write( fd, ptr, size*nmemb )/size;
}
  
int	NnmFClose( int fd )
{
  return close( fd );
}

long	NnmFSeek( int fd, long offset, int whence )
{
  return lseek( fd, offset, whence );
}

long	NnmFTell( int fd )
{
  return lseek( fd, 0L, SEEK_CUR );
}

int	NnmFEof( int fd )
{
  int	res;
  long	cur_ptr, last_ptr;
  
  cur_ptr  = NnmFTell( fd );
  last_ptr = NnmFSeek( fd, 0L, SEEK_END );
  res =  ( cur_ptr >= last_ptr )? 1:0;
  NnmFSeek( fd, cur_ptr, SEEK_SET );
  return res;
}

int	NnmReadStandardFeature( int fd, NnmFeatureDicRec *fv )
{
  int	dirfv_size = 8*NnmFV_REGION_Y*NnmFV_REGION_X;
  int	res[10];

  res[0] = NnmFRead( &(fv->code),       sizeof(ulong), 1, fd );
  res[1] = NnmFRead( &(fv->num_sample), sizeof(ulong), 1, fd );
  res[2] = NnmFRead( &(fv->Cand.sizeX), sizeof(uchar), 1, fd );
  res[3] = NnmFRead( &(fv->Cand.sizeY), sizeof(uchar), 1, fd );
  res[4] = NnmFRead( &(fv->Cand.num_stroke), sizeof(uchar), 1, fd );
  res[5] = NnmFRead( fv->dir_fv,        sizeof(uchar)*dirfv_size, 1, fd );
#ifdef NNM_WEIGHT
  res[6] = NnmFRead( fv->weight, sizeof(uchar)*dirfv_size, 1, fd );
#endif
  res[7] = NnmFRead( fv->Cand.dir_stroke, sizeof(uchar)*8, 1, fd );
#ifdef NNM_WEIGHT
  res[8] = NnmFRead( fv->Cand.dir_stroke_weight, sizeof(uchar)*8, 1, fd );
#endif
#ifdef CAND_TEST
  res[9] = NnmFRead( fv->Cand.strkLong,
		     sizeof(uchar)*8*NnmCAND_SPLIT_NUM, 1, fd );
#ifdef CAND_TEST2
  res[9] = NnmFRead( fv->Cand.strkShort,
		     sizeof(uchar)*8*NnmCAND_SPLIT_NUM*NnmCAND_SPLIT_NUM,
		     1, fd );
#else
  res[9] = NnmFRead( fv->Cand.strkShort,
		     sizeof(uchar)*8*NnmCAND_SPLIT_NUM, 1, fd );
#endif
#endif
#ifdef DEBUG_PRINT
  fprintf(stderr,"Code %5x: num %3d: (%4d x %4d) : stroke %3d : Read\n",
	 fv->code, fv->num_sample, fv->Cand.sizeX, fv->Cand.sizeY,
	 fv->Cand.num_stroke);
#endif
  return 0;
}


int	NnmWriteStandardFeature( int fd, NnmFeatureDicRec *fv )
{
  int	dirfv_size = 8*NnmFV_REGION_Y*NnmFV_REGION_X;

#ifdef DEBUG_PRINT
  fprintf(stderr,"Code %5x: num %3d: (%4d x %4d) : stroke %3d : Write\n",
	 fv->code, fv->num_sample, fv->Cand.sizeX, fv->Cand.sizeY,
	 fv->Cand.num_stroke);
#endif
  NnmFWrite( &(fv->code), sizeof(ulong), 1, fd );
  NnmFWrite( &(fv->num_sample), sizeof(ulong), 1, fd );
  NnmFWrite( &(fv->Cand.sizeX), sizeof(uchar), 1, fd );
  NnmFWrite( &(fv->Cand.sizeY), sizeof(uchar), 1, fd );
  NnmFWrite( &(fv->Cand.num_stroke), sizeof(uchar), 1, fd );
  NnmFWrite( fv->dir_fv, sizeof(uchar)*dirfv_size, 1, fd );
#ifdef NNM_WEIGHT
  NnmFWrite( fv->weight, sizeof(uchar)*dirfv_size, 1, fd );
#endif
  NnmFWrite( fv->Cand.dir_stroke, sizeof(uchar)*8, 1, fd );
#ifdef NM_WEIGHT
  NnmFWrite( fv->Cand.dir_stroke_weight, sizeof(uchar)*8, 1, fd );
#endif
#ifdef CAND_TEST
  NnmFWrite( fv->Cand.strkLong,
	     sizeof(uchar)*8*NnmCAND_SPLIT_NUM, 1, fd );
#ifdef CAND_TEST2
  NnmFWrite( fv->Cand.strkShort,
	     sizeof(uchar)*8*NnmCAND_SPLIT_NUM*NnmCAND_SPLIT_NUM,
	     1, fd );
#else
  NnmFWrite( fv->Cand.strkShort,
	     sizeof(uchar)*8*NnmCAND_SPLIT_NUM, 1, fd );
#endif
#endif
  return 0;
}
