/* 
   $Id: nnmDicman.cpp,v 1.1.1.1 2005/06/29 22:24:22 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

/* Dictinary maintenance tool */

#include	<qpeapplication.h>
#include	<qlabel.h>
#include	<qpainter.h>
#include	<qpixmap.h>
#include	<qwidget.h>
#include	<qlayout.h>
#include	<qpushbutton.h>
#include	<qlineedit.h>

#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<signal.h>

extern "C" {
#include	"Nnm.h"
#include	"NnmFeature.h"
#include	"nnmStub.h"
#include	"NnmProto.h"
}
#include	"nnmCanvas.h"

class uiWidget : public QWidget
{
Q_OBJECT

public:
	uiWidget( QWidget *parent = 0, const char *name = 0, WFlags f=0 );

	int		serverPID;
	char		*dictionaryPath;

private slots:
	void	regist( void );
	void	clear( void );
	void	quit( void );

	void	format( const QString & );

private:
	nnmCanvas	*canvas;
	QLineEdit	*input;
	ulong		code;

	void		updateDictionary( void );
};

#include	"moc_nnmDicman.cpp"

uiWidget::uiWidget( QWidget *parent, const char *name, WFlags f )
			: QWidget( parent, name, f )
{
	setCaption( tr( "DictonaryMaintenance" ) );
	setFixedSize( 240, 128 );

	code = 0;

	QBoxLayout *topLayout	= new QHBoxLayout( this );
	QBoxLayout *left	= new QHBoxLayout( topLayout );
	QBoxLayout *right	= new QVBoxLayout( topLayout );
	QBoxLayout *head	= new QHBoxLayout( right );

	canvas = new nnmCanvas( this, 0, 0 );
	left->addWidget( canvas );

	QLabel *label = new QLabel( QString( tr( "Code : " ) ), this );
	head->addWidget( label );

	input = new QLineEdit( this );
	input->setFixedSize( 24, 20 );
	input->setMaxLength( 2 );
	connect( input, SIGNAL( textChanged( const QString & ) ),
		 this, SLOT( format( const QString & ) ) );
	head->addWidget( input, 20 );

	QPushButton *btn1 = new QPushButton( this );
	btn1->setFixedSize( 96, 20 );
	btn1->setText( tr( "Regist" ) );
	connect( btn1, SIGNAL( clicked() ), this, SLOT( regist() ) );
	right->addWidget( btn1, 20 );

	QPushButton *btn2 = new QPushButton( this );
	btn2->setFixedSize( 96, 20 );
	btn2->setText( tr( "Clear" ) );
	connect( btn2, SIGNAL( clicked() ), this, SLOT( clear() ) );
	right->addWidget( btn2, 20 );

	QPushButton *btn3 = new QPushButton( this );
	btn3->setFixedSize( 96, 20 );
	btn3->setText( tr( "Exit" ) );
	connect( btn3, SIGNAL( clicked() ), this, SLOT( quit() ) );
	right->addWidget( btn3, 20 );

	topLayout->activate();
}

void	uiWidget::regist( void )
{
	if ( canvas->isNull()  ||  code == 0 )	{
		return;
	}

	updateDictionary();

	canvas->init();
	code = 0;
	input->setText( "" );

	if ( serverPID )	{
		kill( serverPID,SIGHUP );
	}
}

void	uiWidget::clear( void )
{
	canvas->init();
}

void	uiWidget::quit( void )
{
	exit( 0 );
}

void	uiWidget::format( const QString &data )
{
	if ( data.length() > 1 )	{
		input->setText( QString( data.at( 1 ) ) );
	} else	{
		code = input->text().at( 0 ).unicode();
	}
}

void	uiWidget::updateDictionary( void )
{
	NnmCharInfoRec		dummy;
	NnmFeatureRec		feature;
	NnmFeatureDicRec	dicData;
	int			fd;
	bool			found = false;
	long			fpos = 0;

	NnmConvertCharInfoToFeature( ( NnmRawCharInfoRec * )canvas->getStroke(),
				     &dummy, &feature );

	fd = NnmFOpen( dictionaryPath, "r+" );
	if ( fd <= 0 )	return;

	while ( ! NnmFEof( fd ) )	{
		fpos = NnmFTell( fd );
		NnmReadStandardFeature( fd, &dicData );
		if ( dicData.code == code )	{
			found = true;
			break;
		}
	}

	if ( found )	{
		NnmUpdateFeatureDic( &feature, &dicData );
		NnmFSeek( fd, fpos, SEEK_SET );
	} else {
		NnmFeatureToFeatureDic( code, &feature, &dicData );
		NnmFSeek( fd, 0, SEEK_END );
	}

	NnmWriteStandardFeature( fd, &dicData );
	NnmFClose( fd );
}

int	main( int argc, char **argv )
{
	if ( argc != 3 )	{
		return -1;
	}

	QPEApplication	myApp( argc, argv );

	myApp.setFont( QString( "Cyberbit" ) );

	uiWidget *widget = new uiWidget( NULL );

	widget->serverPID = atoi( *++argv );
	widget->dictionaryPath = *++argv;

	myApp.setMainWidget( widget );
	widget->show();
	return myApp.exec();
}
