/* 
   $Id: nnmServer.c,v 1.1.1.1 2005/06/29 22:24:34 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <stdio.h>
#include <signal.h>
#include "Nnm.h"
#include "NnmProto.h"
#include "nnmServer.h"

static char *readString(char *buf, unsigned int buflen)
{
    unsigned short len;

    if (fread(&len,sizeof(unsigned short),1,stdin) <= 0) {
	return NULL;
    }
    if (len >= buflen) return NULL;
    if (fread(buf,sizeof(char),len,stdin) < len) {
	return NULL;
    }
    buf[len] = '\0';
    return buf;
}

static void callSetDictionaryFile()
{
    char *s;
    char strbuf[NNM_STRBUF_SIZE];
    int user;

    if ((s = readString(strbuf,NNM_STRBUF_SIZE)) == NULL) {
	fprintf(stderr,"callSetDictionaryFile:Can't read dictionary file name\n");
	exit(2);
    }
    if (fread(&user,sizeof(int),1,stdin) <= 0) {
	fprintf(stderr,"callSetDictionaryFile:Can't read user\n");
	exit(2);
    }
    NnmSetDictionaryFile(s,user);
}

static void callRecognizeCharInfo()
{
    NnmRawCharInfoRec charInfo;
    NnmCandidateRec cands[MAX_CANDIDATE_CATEGORY];
    NnmFeatureRec fv;
    unsigned char num_cands;
    ulong code;
    static unsigned char mno = NNM_RECOGNIZE_CHAR_INFO;
    int i;

    if (fread(&charInfo,sizeof(NnmRawCharInfoRec),1,stdin) <= 0) {
	fprintf(stderr,"callRecognizeCharInfo:Can't read charInfo\n");
	exit(2);
    }
    if (fread(&num_cands,sizeof(unsigned char),1,stdin) <= 0) {
	fprintf(stderr,"callRecognizeCharInfo:Can't read num_cands\n");
	exit(2);
    }
    if (num_cands > MAX_CANDIDATE_CATEGORY) {
	fprintf(stderr,"callRecognizeCharInfo:too large num_cands\n");
	exit(2);
    }
    code = NnmRecognizeCharInfo(&charInfo,cands,num_cands,&fv);

    if (fwrite(&mno,sizeof(unsigned char),1,stdout) <= 0) {
	fprintf(stderr,"callRecognizeCharInfo:Can't write metod No\n");
	exit(2);
    }
    if (fwrite(&num_cands,sizeof(unsigned char),1,stdout) <= 0) {
	fprintf(stderr,"callRecognizeCharInfo:Can't write num_cands\n");
	exit(2);
    }
    for (i = 0;i < num_cands;i++) {
	if (fwrite(&(cands[i].code),sizeof(ulong),1,stdout) < 1) {
	    fprintf(stderr,"callRecognizeCharInfo:Can't write cands\n");
	    exit(2);
	}
    }
    if (fwrite(&fv,sizeof(NnmFeatureRec),1,stdout) <= 0) {
	fprintf(stderr,"callRecognizeCharInfo:Can't write FeatureRec\n");
	exit(2);
    }
    if (fwrite(&code,sizeof(ulong),1,stdout) <= 0) {
	fprintf(stderr,"callRecognizeCharInfo:Can't write code\n");
	exit(2);
    }
    fflush(stdout);
}

static void callSampleAppendFile()
{
    NnmRawCharInfoRec rawCharInfo;
    ulong code;
    char strbuf[NNM_STRBUF_SIZE];
    char *fn;

    if (fread(&rawCharInfo,sizeof(NnmRawCharInfoRec),1,stdin) <= 0) {
	fprintf(stderr,"callSampleAppedFile:Can't read canvasInfo\n");
	exit(2);
    }
    if (fread(&code,sizeof(ulong),1,stdin) <= 0) {
	fprintf(stderr,"callSampleAppedFile:Can't read code\n");
	exit(2);
    }
    if ((fn = readString(strbuf,NNM_STRBUF_SIZE)) == NULL) {
	fprintf(stderr,"callSampleAppedFile:Can't read filename\n");
	exit(2);
    }
    NnmSampleAppendFile(&rawCharInfo,code,fn);
}

static void callUpdateUserDictionary()
{
    NnmFeatureRec fv;
    ulong code;

    if (fread(&code,sizeof(ulong),1,stdin) <= 0) {
	fprintf(stderr,"callUpdateUserDictionary:Can't read code\n");
	exit(2);
    }
    if (fread(&fv,sizeof(NnmFeatureRec),1,stdin) <= 0) {
	fprintf(stderr,"callUpdateUserDictionary:Can't read FeatureRec\n");
	exit(2);
    }
    NnmUpdateUserDictionary(code,&fv);
}

static int requestRefresh = 0;

static void handleHup(int signo)
{
    requestRefresh = 1;
}

int main(int argc, char **argv)
{
    unsigned char method;
    struct sigaction act;

    act.sa_handler = handleHup;
    sigemptyset(&(act.sa_mask));
    act.sa_flags = 0;
    sigaction(SIGHUP,&act,NULL);
    for (;;) {
	while (fread(&method, sizeof(unsigned char),1,stdin) > 0) {
	    switch (method) {
	    case NNM_SET_DICTIONARY_FILE:
		callSetDictionaryFile();
		break;
	    case NNM_RECOGNIZE_CHAR_INFO:
		callRecognizeCharInfo();
		break;
	    case NNM_SAMPLE_APPEND_FILE:
		callSampleAppendFile();
		break;
	    case NNM_UPDATE_USER_DICTIONARY:
		callUpdateUserDictionary();
		break;
	    default:
		fprintf(stderr,"Unknown method\n");
		exit(2);
		break;
	    }
	}
	if (requestRefresh) {
	    while (requestRefresh) {
		requestRefresh = 0;
		NnmRefreshDicCache();
	    }
	} else {
	    break;
	}
    }
    return 0;
}
