/* 
   $Id: nnmCanvas.cpp,v 1.1.1.1 2005/06/29 22:24:41 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* The Canvas for stroke drawing */

#include	<qpainter.h>
#include	<qpixmap.h>
#include	<qwidget.h>
#include	<qframe.h>

#include	<stdio.h>

#include	"Nnm.h"
#include	"nnmCanvas.h"
#include	"moc_nnmCanvas.cpp"

nnmCanvas::nnmCanvas( QWidget *parent, int timeoutParam, int initID,
		      const char *name, WFlags f )
			: QWidget( parent, name, f )
{
	setFixedSize( NnmCANVAS_DISPLAY_SIZE, NnmCANVAS_DISPLAY_SIZE );
	init();
	setBackgroundMode( NoBackground );

	id = initID;
	timeoutMsec = timeoutParam;
	if ( timeoutMsec > 0 )	{
		timer = new QTimer( this );
		connect( timer, SIGNAL( timeout() ),
			 this,  SLOT( timeoutRecognize() ) );
	}
}

void	nnmCanvas::timeoutRecognize( void )
{
	emit eventHappen( id, EVENT_TIMEOUT );
}

bool	nnmCanvas::isNull( void )
{
	return rawData.num == 0;
}

void	nnmCanvas::mousePressEvent( QMouseEvent *event )
{
	if ( rawData.num >= NnmMaxSTROKE_NUM )	return;

	if ( timeoutMsec > 0  &&  timer->isActive() )	{
		timer->stop();
	}

	if ( rawData.num == 0 )	{
		emit eventHappen( id, EVENT_PEN_DOWN );
	}

	if ( rawData.stroke[rawData.num].len > 0 )	rawData.num++;

	NnmRawStrokeRec *stroke_p = &rawData.stroke[rawData.num];

	stroke_p->sample_x[0] = ( uchar )event->x();
	stroke_p->sample_y[0] = ( uchar )event->y();
	stroke_p->len = 1;
}

void	nnmCanvas::mouseMoveEvent( QMouseEvent *event )
{
	QPainter	winPainter;
	QPainter	bufPainter;

	if ( rawData.num >= NnmMaxSTROKE_NUM )		return;

	NnmRawStrokeRec *stroke_p = &rawData.stroke[rawData.num];

	if ( stroke_p->len >= NnmMaxSTROKE_LEN )	return;

	stroke_p->sample_x[stroke_p->len] = ( uchar )event->x();
	stroke_p->sample_y[stroke_p->len] = ( uchar )event->y();
	stroke_p->len++;

	if ( stroke_p->len > 1 )	{
		QPoint last = QPoint(
				( int )stroke_p->sample_x[stroke_p->len-2],
				( int )stroke_p->sample_y[stroke_p->len-2] );

		winPainter.begin( this );
		bufPainter.begin( &_buffer );

		winPainter.drawLine( last, event->pos() );
		bufPainter.drawLine( last, event->pos() );

		winPainter.end();
		bufPainter.end();
	}
}

void	nnmCanvas::mouseReleaseEvent( QMouseEvent *event )
{
	if ( rawData.num >= NnmMaxSTROKE_NUM )		return;

	if ( timeoutMsec > 0  &&  ( ! timer->isActive() ) )	{
		timer->start( timeoutMsec, true );
	}

	rawData.num++;
}

void	nnmCanvas::paintEvent( QPaintEvent *event )
{
	QPainter	bufPainter;
  
	bufPainter.begin( &_buffer );

	bufPainter.drawLine(  0,  0, 63,  0 );
	bufPainter.drawLine( 63,  0, 63, 63 );
	bufPainter.drawLine( 63, 63,  0, 63 );
	bufPainter.drawLine(  0, 63,  0,  0 );

	bufPainter.end();

	bitBlt( this, 0, 0, &_buffer );
}

void	nnmCanvas::resizeEvent( QResizeEvent *event )
{
	QPixmap	save( _buffer );
	_buffer.resize( event->size() );
	_buffer.fill( white );
	bitBlt( &_buffer, 0, 0, &save );
}

void	nnmCanvas::init( void )
{
	if ( timeoutMsec > 0  &&  timer->isActive() )	{
		timer->stop();
	}

	_buffer.fill( white );
	repaint();

	memset( &rawData, 0, sizeof ( rawData ) );
	rawData.height = NnmCANVAS_DISPLAY_SIZE;
	rawData.width  = NnmCANVAS_DISPLAY_SIZE;
}

const NnmRawCharInfoRec	*nnmCanvas::getStroke( void )
{
	return ( const NnmRawCharInfoRec * )&rawData;
}
