/* 
   $Id: nnmStub.c,v 1.1.1.1 2005/06/29 22:24:41 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "config.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "Nnm.h"
#include "NnmFeature.h"
#include "../server/nnmServer.h"

static FILE *ofp;
static FILE *ifp;

void NnmStubInit(FILE *op, FILE *ip)
{
    ofp = op;
    ifp = ip;
}

static void writeString(char *s)
{
    unsigned short len;

    len = strlen(s);
    if (len >= NNM_STRBUF_SIZE) {
	fprintf(stderr,"writeString: too long string len\n");
	exit(2);
    }
    if (fwrite(&len,sizeof(unsigned short),1,ofp) <= 0) {
	fprintf(stderr,"writeString: Can't write len\n");
	exit(2);
    }
    if (fwrite(s,sizeof(char),len,ofp) < len) {
	fprintf(stderr,"writeString: Can't write string body\n");
	exit(2);
    }
}

void NnmStubSetDictionaryFile(char *fn, int user)
{
    static unsigned char mno = NNM_SET_DICTIONARY_FILE;

    if (fwrite(&mno,sizeof(unsigned char),1,ofp) <= 0) {
	fprintf(stderr,"StubSetDictionaryFile:Can't write metod No\n");
	exit(2);
    }
    writeString(fn);
    if (fwrite(&user,sizeof(int),1,ofp) <= 0) {
	fprintf(stderr,"StubSetDictionaryFile:Can't write user\n");
	exit(2);
    }
    fflush(ofp);
}

void NnmStubUpdateUserDictionary(long code, NnmFeatureRec *fv)
{
    static unsigned char mno = NNM_UPDATE_USER_DICTIONARY;

    if (fwrite(&mno,sizeof(unsigned char),1,ofp) <= 0) {
	fprintf(stderr,"StubUpdateUserDictionary:Can't write metod No\n");
	exit(2);
    }
    if (fwrite(&code,sizeof(long),1,ofp) <= 0) {
	fprintf(stderr,"StubUpdateUserDictionary:Can't write code\n");
	exit(2);
    }
    if (fwrite(fv,sizeof(NnmFeatureRec),1,ofp) <= 0) {
	fprintf(stderr,"StubUpdateUserDictionary:Can't write FeatureRec\n");
	exit(2);
    }
    fflush(ofp);
}

void NnmStubRecognizeCharInfo(NnmRawCharInfoRec *charInfo,
  int num_cands)
{
    static unsigned char mno = NNM_RECOGNIZE_CHAR_INFO;
    unsigned char n = num_cands;

    if (fwrite(&mno,sizeof(unsigned char),1,ofp) <= 0) {
	fprintf(stderr,"StubSetRecognizeCharInfo:Can't write metod No\n");
	exit(2);
    }
    if (fwrite(charInfo,sizeof(NnmRawCharInfoRec),1,ofp) <= 0) {
	fprintf(stderr,"StubSetRecognizeCharInfo:Can't write charInfo\n");
	exit(2);
    }
    if (fwrite(&n,sizeof(unsigned char),1,ofp) <= 0) {
	fprintf(stderr,"StubSetRecognizeCharInfo:Can't write num_cands\n");
	exit(2);
    }
    fflush(ofp);
}

void NnmStubSampleAppendFile(NnmRawCharInfoRec *rawCharInfo, ulong code,
  char *fn)
{
    static unsigned char mno = NNM_SAMPLE_APPEND_FILE;

    if (fwrite(&mno,sizeof(unsigned char),1,ofp) <= 0) {
	fprintf(stderr,"StubSampleAppendFile:Can't write metod No\n");
	exit(2);
    }
    if (fwrite(rawCharInfo,sizeof(NnmRawCharInfoRec),1,ofp) <= 0) {
	fprintf(stderr,"StubSampleAppendFile:Can't write canvasInfo\n");
	exit(2);
    }
    if (fwrite(&code,sizeof(ulong),1,ofp) <= 0) {
	fprintf(stderr,"StubSampleAppendFile:Can't write code\n");
	exit(2);
    }
    writeString(fn);
    fflush(ofp);
}

ulong NnmStubResultRecognizeCharInfo(ulong *cands, int num_cands,
  NnmFeatureRec *fv)
{
    unsigned char mno;
    unsigned char n;
    ulong code;

    if (fread(&mno,sizeof(unsigned char),1,ifp) <= 0) {
	fprintf(stderr,"ResultRecognizeCharInfo:Can't read metod No\n");
	exit(2);
    }
    if (mno != NNM_RECOGNIZE_CHAR_INFO) {
	fprintf(stderr,"ResultRecognizeCharInfo:method No mismatch\n");
	exit(2);
    }
    if (fread(&n,sizeof(unsigned char),1,ifp) <= 0) {
	fprintf(stderr,"ResultRecognizeCharInfo:Can't read num_cands\n");
	exit(2);
    }
    if (num_cands != n) {
	fprintf(stderr,"ResultRecognizeCharInfo:num_cands mismatch\n");
	exit(2);
    }
    if (fread(cands,sizeof(ulong),num_cands,ifp) < num_cands) {
	fprintf(stderr,"ResultRecognizeCharInfo:Can't read cands\n");
	exit(2);
    }
    if (fread(fv,sizeof(NnmFeatureRec),1,ifp) <= 0) {
	fprintf(stderr,"ResultRecognizeCharInfo:Can't read FeatureRec\n");
	exit(2);
    }
    if (fread(&code,sizeof(ulong),1,ifp) <= 0) {
	fprintf(stderr,"ResultRecognizeCharInfo:Can't read code\n");
	exit(2);
    }
    return code;
}
