/* 
   $Id: nunome.cpp,v 1.1.1.1 2005/06/29 22:24:43 leggewie Exp $

   Copyright (C) 2001-2002 AXE,Inc. <sg@axe-inc.co.jp>

   This file is part of Nunome-Qtopia.

   Nunome-Qtopia is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Nunome-Qtopia is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Nunome; see the file COPYING.   If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Nunome input-method */

#include	<qpeapplication.h>
#include	<qpainter.h>
#include	<qpixmap.h>
#include	<qwidget.h>
#include	<qlayout.h>
#include	<qpushbutton.h>
#include	<qbuttongroup.h>

#include	<stdio.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<ctype.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>

#include	"Nnm.h"
extern "C" {
#include	"nnmStub.h"
}
#include	"nnmCanvas.h"
#include	"nunome.h"

/* Start Nunome server */
pid_t	Nunome::startServer( char *path )
{
	int	infd[2];
	int	outfd[2];
	FILE	*ifp,*ofp;

	if ( pipe(infd) < 0 )	{
		fprintf( stderr, "pipe fail\n" );
		exit( 2 );
	}
	if ( pipe(outfd) < 0 )	{
		fprintf( stderr, "pipe fail\n" );
		exit( 2 );
	}

	pid_t pid = fork();
	if ( pid == 0 )	{
		::close( infd[0] );
		::close( outfd[1] );
		dup2( infd[1], 1 );
		dup2( outfd[0], 0 );
		execlp( path, path, 0 );
		fprintf( stderr, "Exec server fail\n" );
		exit( 2 );
	}

	::close( infd[1] );
	::close( outfd[0] );
	if ( ( ifp = fdopen(infd[0], "r" ) ) == NULL )	{
		fprintf( stderr, "fdopen fail\n" );
		exit( 2 );
	}
	if ( ( ofp = fdopen(outfd[1],"w" ) ) == NULL )	{
		fprintf( stderr, "fdopen fail\n" );
		exit( 2 );
	}
	NnmStubInit( ofp, ifp );

	QSocketNotifier	*notifier = new QSocketNotifier( infd[0],
							 QSocketNotifier::Read
						       );
	connect( notifier, SIGNAL( activated( int ) ),
			   SLOT( recognizeDone() ) );

	return pid;
}

// Create a function button
static QPushButton	*createFunctionButton( Nunome *parent )
{
	QPushButton *btn = new QPushButton( parent );
	btn->setFixedSize( 48, 18 );

	return btn;
}

// fgets() like read line
static char	*getLine( char *buff, size_t size, FILE *fp )
{
	if ( ! fgets( buff, size, fp ) )	return NULL;

	int	len = strlen( buff );
	if ( *( buff+len-1 ) != '\n' )	{
		*buff = '\0';
	} else	{
		*( buff+len-1 ) = '\0';
	}

	return buff;
}

// strtok() like get token
static char	*getToken( char *start )
{
	char		*ret;
	static char	*keep;
	size_t		len;

	if ( start != NULL )	{
		start += strspn( ( const char * )start, " \t" );
		len   = strcspn( ( const char * )start, " \t" );
		*( start + len ) = '\0';
		keep = start + len + 1;

		ret = start;
		if ( *start == '#'  ||  len == 0 )	ret = NULL;
	} else	{
		keep += strspn( ( const char * )keep, " \t" );

		ret = keep;
	}

	return ret;
}

// string to upper
static char	*strUpr( char *source )
{
	char	*keep = source;

	while ( *source )	{
		*source = toupper( *source );
		source++;
	}

	return keep;
}

// Set parameter
nnmParameter::nnmParameter( void )
{
	FILE		*fp;
	char		buff[MAX_STR_LEN];
	char		*keyword;
	char		*value;
	struct stat	st;

	/* Set default parameter */
	systemDictionary	= DEFAULT_SYSTEM_DICTIONARY;
	userDictionary		= DEFAULT_USER_DICTIONARY;
	timeoutMsec		= DEFAULT_TIMEOUT_MSEC;
	learnFlag		= DEFAULT_LEARN_FLAG;
	serverProgramPath	= DEFAULT_SERVER_PROGRAM_PATH;
	learnProgramPath	= DEFAULT_LEARN_PROGRAM_PATH;

	/* Read parameter file */
	fp = fopen( PARAMETER_FILE_PATH, "r" );
	if ( fp != NULL )	{
		while ( getLine( buff, sizeof ( buff ), fp ) )	{
			if ( ! buff[0] )	continue;
			keyword = getToken( buff );
			if ( ! keyword )	continue;
			value   = getToken( NULL );
			if ( ! value   )	continue;

			if ( ! strcmp( keyword, "systemDictionary" ) )	{
				systemDictionary = strdup( value );
				continue;
			}
			if ( ! strcmp( keyword, "userDictionary" ) )	{
				userDictionary = strdup( value );
				continue;
			}
			if ( ! strcmp( keyword, "timeoutMsec" ) )	{
				timeoutMsec = atoi( value );
				continue;
			}
			if ( ! strcmp( keyword, "learnFlag" ) )	{
				if ( strcmp( strUpr( value ), "TRUE" ) == 0  ||
				     strcmp( strUpr( value ), "ON"   ) == 0  ||
				     strcmp( value, "1" ) )	{
					learnFlag = true;
				} else	{
					learnFlag = false;
				}
				continue;
			}
			if ( ! strcmp( keyword, "serverProgramPath" ) )	{
				serverProgramPath = strdup( value );
				continue;
			}
			if ( ! strcmp( keyword, "learnProgramPath" ) )	{
				learnProgramPath = strdup( value );
				continue;
			}
		}
	}

	/* When not exist user-dictionary */
	if ( stat( userDictionary,&st ) < 0 )	{
		if ( creat( userDictionary,0666 ) < 0 )	{
			fprintf( stderr,
				"Can't create User dictionary:%s\n",
				userDictionary );
			exit( 2 );
		}
	}
}

// Constructor
Nunome::Nunome( QWidget *parent, const char *name, WFlags f )
			: QFrame( parent, name, f )
{
	QString	file = QPEApplication::qpeDir() + "/i18n/" + getenv( "LANG" ) + "/nunome.qm";
	QTranslator	translator( 0 );
	translator.load( file );
	qApp->installTranslator( &translator );

	parameter = nnmParameter();

	serverPID = startServer( parameter.serverProgramPath );

#define	SYSTEM_DIC	0
#define	USER_DIC	1
	NnmStubSetDictionaryFile( parameter.systemDictionary, SYSTEM_DIC );
	NnmStubSetDictionaryFile( parameter.userDictionary,   USER_DIC );

	setFrameStyle( Panel | Sunken );

	QBoxLayout *topLayout	= new QHBoxLayout( this );
	QBoxLayout *left	= new QVBoxLayout( topLayout );
	QGridLayout *right	= new QGridLayout( topLayout, 4, 2 );
	QBoxLayout *candidateBox = new QHBoxLayout( left );
	QBoxLayout *canvasBox	= new QHBoxLayout( left );

	canvas[0] = new nnmCanvas( this, parameter.timeoutMsec, 0 );
	canvas[1] = new nnmCanvas( this, parameter.timeoutMsec, 1 );
	canvasBox->addWidget( canvas[0] );
	canvasBox->addWidget( canvas[1] );
	connect( canvas[0], SIGNAL( eventHappen( int, int ) ),
		 this,      SLOT( startRecognizeByCanvas( int, int ) ) );
	connect( canvas[1], SIGNAL( eventHappen( int, int ) ),
		 this,      SLOT( startRecognizeByCanvas( int, int ) ) );

	lookingCanvas = NULL;

	QButtonGroup *candidateGroup = new QButtonGroup( this );
	candidateGroup->hide();

	for ( int i = 0; i < MAX_DISP_RESULT; i++ )	{
		candidate[i] = new nnmCandidate( this );
		candidateBox->addWidget( candidate[i] );
		candidateGroup->insert( candidate[i]->button );
	}

	connect( candidateGroup, SIGNAL( clicked( int ) ),
		 this, SLOT( startSelect( int ) ) );

	// Remove all candidates
	QPushButton *btn00 = createFunctionButton( this );
	btn00->setText( tr( "Remove" ) );
	connect( btn00, SIGNAL( clicked() ), this, SLOT( clearCandidate() ) );
	right->addWidget( btn00, 0, 0 );

	// Send Enter key code
	QPushButton *btn10 = createFunctionButton( this );
	btn10->setText( tr( "Enter" ) );
	connect( btn10, SIGNAL( clicked() ), this, SLOT( sendEnter() ) );
	right->addWidget( btn10, 1, 0 );

	// Start recognize
	QPushButton *btn20 = createFunctionButton( this );
	btn20->setText( tr( "Recog" ) );
	connect( btn20, SIGNAL( clicked() ), this, SLOT( startRecognize() ) );
	right->addWidget( btn20, 2, 0 );

	// Send Backspace key code
	QPushButton *btn30 = createFunctionButton( this );
	btn30->setText( tr( "BS" ) );
	connect( btn30, SIGNAL( clicked() ), this, SLOT( sendBackspace() ) );
	right->addWidget( btn30, 3, 0 );

	// Send Space key code
	QPushButton *btn01 = createFunctionButton( this );
	btn01->setText( tr( "Space" ) );
	connect( btn01, SIGNAL( clicked() ), this, SLOT( sendSpace() ) );
	right->addWidget( btn01, 0, 1 );

	// Send all candidates
	QPushButton *btn11 = createFunctionButton( this );
	btn11->setText( tr( "Done" ) );
	connect( btn11, SIGNAL( clicked() ), this, SLOT( sendAll() ) );
	right->addWidget( btn11, 1, 1 );

	// Start to regist
	QPushButton *btn21 = createFunctionButton( this );
	btn21->setText( tr( "Resist" ) );
	connect( btn21, SIGNAL( clicked() ), this, SLOT( startRegist() ) );
	right->addWidget( btn21, 2, 1 );

	// Start to select symbols
	QPushButton *btn31 = createFunctionButton( this );
	btn31->setText( tr( "Symbol" ) );
	connect( btn31, SIGNAL( clicked() ), this, SLOT( startSymbol() ) );
	right->addWidget( btn31, 3, 1 );

	// Create select panel
	selectPanel = new nnmSelect( this );

	// Create symbol select panel
	symbolPanel = new nnmSymbol( this );
	connect( symbolPanel, SIGNAL( selected() ),
		 this, SLOT( sendSymbol() ) );

	topLayout->activate();
}

void	Nunome::resetState( void )
{
	clearCandidate();
}

void	Nunome::clearCandidate( void )
{
	selectPanel->hide();

	for ( int i = 0; i < MAX_DISP_RESULT; i++ )	{
		candidate[i]->init();
	}
}

void	Nunome::sendEnter( void )
{
	selectPanel->hide();

	sendAll();
	emit sendCode( 0, Qt::Key_Enter, 0, true,  true );
	emit sendCode( 0, Qt::Key_Enter, 0, false, true );
}

void	Nunome::sendBackspace( void )
{
	selectPanel->hide();

	for ( int i = MAX_DISP_RESULT - 1; i >= 0; i-- )	{
		if ( ! candidate[i]->isNull() )	{
			candidate[i]->init();
			return;
		}
	}

	emit sendCode( 0, Qt::Key_Backspace, 0, true,  false );
	emit sendCode( 0, Qt::Key_Backspace, 0, false, false );
}

void	Nunome::sendSpace( void )
{
	selectPanel->hide();

	emit sendCode( ' ', Qt::Key_Space, 0, true,  false );
	emit sendCode( ' ', Qt::Key_Space, 0, false, false );
}

void	Nunome::sendAll( void )
{
	selectPanel->hide();

	for ( int i = 0; i < MAX_DISP_RESULT; i++ )	{
		if ( candidate[i]->isNull() )	continue;

		emit sendCode( candidate[i]->suitable, 0, 0, true,  false );
		emit sendCode( candidate[i]->suitable, 0, 0, false, false );

		if ( parameter.learnFlag &&
		     candidate[i]->suitable != candidate[i]->list[0] )	{
			NnmStubUpdateUserDictionary( candidate[i]->suitable,
						    &candidate[i]->feature );
		}

		candidate[i]->init();
	}
}

void	Nunome::startSelect( int pos )
{
	if ( ! candidate[pos]->isNull() )	{
		selectPanel->start( candidate[pos] );
	}
}

void	Nunome::startRegist( void )
{
	char	work[MAX_STR_LEN];

	if ( ! parameter.learnFlag )	return;

	selectPanel->hide();

	int len = snprintf( work, sizeof ( work ), "%s %d %s &",
			parameter.learnProgramPath,
			serverPID,
			parameter.userDictionary );
	if ( len < 0  ||  work[len -1] != '&' )	return;

	system( work );
}

void	Nunome::startSymbol( void )
{
	selectPanel->hide();

	symbolPanel->start();
}

void	Nunome::sendSymbol( void )
{
	emit sendCode( symbolPanel->code, 0, 0, true,  false );
	emit sendCode( symbolPanel->code, 0, 0, false, false );
}

void	Nunome::startRecognize( void )
{
	selectPanel->hide();

	if ( lookingCanvas )	{
		recognize( lookingCanvas );
	}
}

void	Nunome::startRecognizeByCanvas( int canvasID, int flag )
{
	selectPanel->hide();

	switch ( flag )	{
	case	EVENT_TIMEOUT :
		recognize( canvas[canvasID] );
		break;

	case	EVENT_PEN_DOWN :
		nnmCanvas	*work;

		work = canvas[canvasID ? 0 : 1];
		if ( ! work->isNull() )	{
			recognize( work );
		}

		lookingCanvas = canvas[canvasID];

		break;
	default :
		break;
	}
}

void	Nunome::shiftCandidates( void )
{
	// hide select panel
	selectPanel->hide();

	// send most left candidate field's code
	emit sendCode( candidate[0]->suitable, 0, 0, true,  false );
	emit sendCode( candidate[0]->suitable, 0, 0, false, false );

	// shift to left
	for ( int i = 0; i < MAX_DISP_RESULT - 1; i++ )	{
		memcpy( candidate[i]->list, candidate[i+1]->list,
			sizeof ( candidate[i]->list ) );
		candidate[i]->setSuitable( candidate[i+1]->suitable );
	}

	// clear most right field
	candidate[MAX_DISP_RESULT - 1]->init();
}

nnmCandidate	*Nunome::getTargetCandidate( void )
{
	for ( int i = 0; i < MAX_DISP_RESULT; i++ )	{
		if ( candidate[i]->isNull() )	return candidate[i];
	}

	shiftCandidates();

	return candidate[MAX_DISP_RESULT - 1];
}

void	Nunome::recognize( nnmCanvas *targetCanvas )
{
	NnmRawCharInfoRec *data = ( NnmRawCharInfoRec * )targetCanvas->getStroke();

	NnmStubRecognizeCharInfo( data, MAX_CANDIDATE_CATEGORY );

	targetCanvas->init();
	lookingCanvas = NULL;
}

void	Nunome::recognizeDone( void )
{
	nnmCandidate *targetCandidate = getTargetCandidate();
	if ( targetCandidate == NULL )	return;

	ulong	result =  NnmStubResultRecognizeCharInfo(
					targetCandidate->list,
					MAX_CANDIDATE_CATEGORY,
					&targetCandidate->feature );

	targetCandidate->setSuitable( result );
}
